<?php

namespace CSPro\Controllers;

use Silex\Application;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Silex\ControllerProviderInterface;
use CSPro\CSProResponse;
use PDO;
use CSPro;
use CSPro\User\UserParser;
use CSPro\User\UserValidator;

class UserController implements ControllerProviderInterface {

    public function connect(Application $app) {
        // creates a new controller based on the default route
        $controllers = $app ['controllers_factory'];
        $controllers->post('/', 'CSPro\Controllers\UserController::addUser')->bind('addUser');
        $controllers->get('/', 'CSPro\Controllers\UserController::getUserList')->bind('getUserList');
        $controllers->get('/{username}', 'CSPro\Controllers\UserController::getUser')->bind('getUser');
        $controllers->put('/{username}', 'CSPro\Controllers\UserController::updateUser')->bind('updateUser');
        $controllers->delete('/{username}', 'CSPro\Controllers\UserController::deleteUser')->bind('deleteUser');

        // check if you can pass a function to the controller for before action
        return $controllers;
    }

    function insertUsers($app, $users) {
        $pdo = $app ['pdo'];

        //$app['monolog']->addDebug('---------------- inside insertUsers');

        $stm = 'INSERT INTO cspro_users (username, password, first_name, last_name, email, phone, role) VALUES ';

        $values = '';
        $numUsers = count($users);

        for ($i = 0; $i < $numUsers; $i++) {
            $user = $users[$i];
            $email = $user->getEmail();
            $phone = $user->getPhone();
            
            $values .= ('(');
            $values .= "'" . $user->getUsername() . "',";
            $values .= "'" . $user->getPassword() . "',";
            $values .= "'" . $user->getFirstName() . "',";
            $values .= "'" . $user->getLastName() . "',";
            if (empty(trim($email))) {
                $values .= "NULL,";
            } else {
                $values .= "'" . $email . "',";
            }
            if (empty(trim($phone))) {
                $values .= "NULL,";
            } else {
                $values .= "'" . $phone . "',";
            }

            $values .= "'" . $user->getRole() . "'";
            $values .= (')');

            if ($i + 1 != $numUsers)
                $values .= ",";
        }

        $stm .= $values;

        $stm .= ' ON DUPLICATE KEY UPDATE password = VALUES(password), first_name = VALUES(first_name), last_name = VALUES(last_name), email = VALUES(email), phone = VALUES(phone), role = VALUES(role)';

        $pdoStm = $pdo->prepare($stm);

        try {
            $result = $pdoStm->execute();
        } catch (\Exception $e) {
            $app['monolog']->addError('Failed adding import users into: ' . 'cspro_user', array('context' => (string) $e));
            throw new \Exception('Failed adding import users into: ' . 'cspro_user', 0, $e);
        }
    }

    // addMultipleUsers
    //TODO: refactor &&&
    function addMultipleUsers(Application $app, Request $request) {
        $app ['monolog']->addDebug("processing users...");
        //ini changes are for the duration of the script and are restored once the script ends
        ini_set('max_execution_time', $app['maxImportScriptExecTime']);
        // Turn off output buffering
        ini_set('output_buffering', 'off');
        // Turn off PHP output compression
        ini_set('zlib.output_compression', false);
        // Implicitly flush the buffer(s)
        ini_set('implicit_flush', true);
        ob_implicit_flush(true);
        // Clear, and turn off output buffering
        while (ob_get_level() > 0) {
            // Get the curent level
            $level = ob_get_level();
            // End the buffering
            ob_end_clean();
            // If the current level has not changed, abort
            if (ob_get_level() == $level)
                break;
        }

        $pdo = $app ['pdo'];
        //create a streamed response
        $response = new StreamedResponse();
        $content = $request->getContent();

        // $app ['monolog']->addDebug("Content: " . $content);

        $headerRow = $request->headers->get('x-csw-data-header');
        isset($headerRow) && $headerRow === "1" ? $headerRow = true : $headerRow = false;
        $userController = $this;
        $parser = new UserParser ();
        $validator = new UserValidator ();
        try {
            // validate import users
            $isValid = $validator->validateImportUsers($app, $content, $headerRow);

            if ($isValid) {
                $users = $parser->parseUsers($app, $content, $headerRow);
            } else {
                $response = new CSProResponse ();
                $strMsg = "";
                foreach ($validator->getErrors() as $error) {
                    $strMsg .= sprintf("%s<br/>", $error);
                }
                $response->setError(400, 'user_file_invalid', $strMsg);
                $response->setStatusCode(400);
                $app ['monolog']->addDebug($strMsg);
                return $response;
            }
        } catch (\Exception $e) {
            $app ['monolog']->addError($e->getMessage());
            $response = new CSProResponse ();
            $response->setError(400, 'user_file_invalid', $e->getMessage());
            $response->setStatusCode(400);
            return $response;
        }

        $app ['monolog']->addDebug("processing users...");
        $response->setCallback(function () use ($app, $request, $users, $parser, $userController, $headerRow) {
            $params = array();
            $userHashMap = array();
            $duplicateMsg = "<br>";
            $duplicateUserCount = 0;
            $responseDescription = "Success";
            //init the block size to 1% of the total number. If it is exceeds the maxblock size then use the max block size. 
            //to give users control for very large imports without having to increase the mysql max_allowed_packet limit
            $blockSize = isset($users) && count($users) ? max(count($users) * 0.01, 50) : 50;
            $blockSize = $blockSize > $app['maxImportBlockSize'] ? $app['maxImportBlockSize'] : $blockSize;

            $app ['monolog']->addDebug('MaxblockSize is: ' . $app['maxImportBlockSize'] . ' Block size is: ' . $blockSize);
            $usersToInsert = array();
            for ($i = 0; $i < count($users); $i++) {
                $user = $parser->transformUser($users[$i]);
                array_push($usersToInsert, $user);

                if (isset($userHashMap[$user->getUserName()])) {
                    $lineNum = ($headerRow === true) ? $i + 2 : $i + 1;
                    $duplicateMsg = $duplicateMsg . "Duplicate user: " . $user->getUserName() . " at line number: " . $lineNum . "<br>";
                    $duplicateUserCount++;
                } else {
                    $userHashMap[$user->getUserName()] = 1;
                }
                //if final block
                if ($i == (count($users) - 1)) {
                    $this->insertUsers($app, $usersToInsert);
                    $percentComplete = 100;
                }
                // else if interim block
                else if ($blockSize == count($usersToInsert)) {

                    $percentComplete = round($i / count($users) * 100);

                    $this->insertUsers($app, $usersToInsert);
                    unset($usersToInsert); // clear $users array
                    $usersToInsert = array(); // clear $users array

                    $responseCode = $percentComplete === 100 ? 200 : 206;
                    if ($percentComplete === 100 && $duplicateUserCount > 0) {
                        $duplicateMsg = $duplicateMsg . "Total duplicate user count is: " . $duplicateUserCount . "<br>";
                        $responseDescription = $responseDescription . $duplicateMsg;
                    }
                    $strJSONResponse = json_encode(array(
                        "code" => $responseCode,
                        "description" => $responseDescription,
                        'progress' => $percentComplete,
                        'count' => $i + 1,
                        'status' => "Success"
                    ));
                    echo '\n' . $strJSONResponse;
                    $app ['monolog']->addDebug($strJSONResponse);

                    flush();
                    $strJSONResponse = ''; //reset json string response
                }
            }
            $responseCode = $percentComplete === 100 ? 200 : 206;
            if ($percentComplete === 100 && $duplicateUserCount > 0) {
                $duplicateMsg = $duplicateMsg . "Total duplicate user count is: " . $duplicateUserCount . "<br>";
                $responseDescription = $responseDescription . $duplicateMsg;
            }
            $strJSONResponse = json_encode(array(
                "code" => $responseCode,
                "description" => $responseDescription,
                'progress' => $percentComplete,
                'count' => $i,
                'status' => "Success"
            ));
            echo '\n' . $strJSONResponse;
            $app ['monolog']->addDebug($strJSONResponse);

            flush();
            $strJSONResponse = ''; //reset json string response
        }
        );
        $response->headers->set('Content-Type', 'application/json');
        return $response->send();
    }

    // addSingleUser
    function addSingleUser(Application $app, Request $request) {

        $params = array();
        $content = $request->getContent();
        $response = new CSProResponse ();
        if (empty($content)) {
            $response->setError(400, 'user_invalid_request', 'Invalid request. Missing JSON content.');
            return $response;
        } else {
            $uri = '#/definitions/User';
            validateEncodedJSON($app, $content, $uri);
            $params = json_decode($content, true); // 2nd param to get as array
            if (!isValidUser($params)) {
                $response->setError(400, 'user_invalid', 'Invalid User Supplied. User attributes cannot be blank or invalid.');
                return $response;
            }
        }
        try {
            $pdo = $app ['pdo'];
            $stm = 'SELECT username  FROM cspro_users WHERE username = :uname;';
            $username = strtolower($params ['username']);
            $bind = array(
                'uname' => array(
                    'uname' => $username
                )
            );
            if ($pdo->fetchValue($stm, $bind)) {
                $response->setError(405, 'user_name_exists', 'Username already in use');
                return $response;
            }

            if (!isset($params ['role'])) {
                $params ['role'] = 1; // Standard User
            }

            $email  = $params ['email'];
            $phone  = $params ['phone'];
            if (empty(trim($email))) {
               $email = null;
            } 
            if (empty(trim($phone))) {
                $phone = null;
            } 
            
            $user = new CSPro\User\User($params['username'], $params['firstName'], $params['lastName'], $params['role'], $params['password'], $email, $phone);
            $parser = new UserParser();
            $user = $parser->transformUser($user);
            $users = array();
            array_push($users, $user);
            $this->insertUsers($app, $users);
        } catch (\Exception $e) {
            $app ['monolog']->addError('Failed adding user: ' . $username, array("context" => (string) $e));
            $response = new CSProResponse();
            $response->setError(500, 'user_add_error', 'Failed adding user');
        }
        $response = new CSProResponse(json_encode(array(
                    "code" => 200,
                    "description" => "Success"
                )), 200);
        $response->headers->set('Content-Length', strlen($response->getContent()));
        return $response;
    }

    // addUser
    function addUser(Application $app, Request $request) {

        $response = new CSProResponse();

        // content type
        $cType = $request->headers->get('Content-Type');
        $app ['monolog']->addDebug("Content Type: " . $cType);
        if ($cType == 'json') {
            $app ['monolog']->addDebug("********JSON contentType:" . $cType . "*******");
            $response = $this->addSingleUser($app, $request);
        } else if ($cType == 'text/plain') {
            $app ['monolog']->addDebug("********TXT contentType:" . $cType . "*******");
            $response = $this->addMultipleUsers($app, $request);
        } else {
            $errMsg = "Failed adding user. Content-Type:" . $cType . " must be either json or txt";
            $app ['monolog']->addError($errMsg);
            $response = new CSProResponse();
            $response->setError(500, 'user_add_error', $errMsg);
        }

        return $response;
    }

    // get users
    function getUserList(Application $app, Request $request) {
        $pdo = $app ['pdo'];
        $userCount = 0;
        $usersFiltered = 0;

        $start = $request->headers->get('x-csw-user-start');
        if ($start == null || $start == "") {
            $start = 0;
        }

        $length = $request->headers->get('x-csw-user-length');
        if ($length == null || $length == "") {
            $length = 1000;
        }

        $search = $request->headers->get('x-csw-user-search');

        $orderColumn = $request->headers->get('x-csw-user-order-column');
        if ($orderColumn == null || $orderColumn == "") {
            $orderColumn = 1;
        } else {
            $orderColumn++; // SQL doesn't use 0 column as the first column
        }

        $orderDirection = $request->headers->get('x-csw-user-order-direction');
        if ($orderDirection == null || $orderDirection == "") {
            $orderDirection = "ASC";
        }

        //users for Table
        try {
            $argArray = array();

            $selectStm = "SELECT username, first_name as firstName, last_name as lastName, email, phone, role as role FROM cspro_users ";

            $searchTF = false;
            if ($search != null && $search != "" && $search != " ") {
                $searchTF = true;
                $searchStm = " WHERE username LIKE :uname OR first_name LIKE :fname OR last_name LIKE :lname OR email LIKE :email OR phone LIKE :phone ";
                $selectStm = $selectStm . $searchStm;
            }

            if (strtolower($orderDirection) == 'asc') {
                $orderByStm = ' ORDER BY :column ASC LIMIT :length OFFSET :start ';
                $selectStm = $selectStm . $orderByStm;
            } else {
                $orderByStm = ' ORDER BY :column DESC LIMIT :length OFFSET :start ';
                $selectStm = $selectStm . $orderByStm;
            }

            //$app ['monolog']->addDebug( '********** query: ' . $selectStm);

            $query = $pdo->prepare($selectStm);

            if ($searchTF) {
                $query->bindValue(':uname', "%" . $search . "%", PDO::PARAM_STR);
                $query->bindValue(':fname', "%" . $search . "%", PDO::PARAM_STR);
                $query->bindValue(':lname', "%" . $search . "%", PDO::PARAM_STR);
                $query->bindValue(':email', "%" . $search . "%", PDO::PARAM_STR);
                $query->bindValue(':phone', "%" . $search . "%", PDO::PARAM_STR);
            }
            $query->bindValue(':column', (int) $orderColumn, PDO::PARAM_INT);
            $query->bindValue(':length', (int) $length, PDO::PARAM_INT);
            $query->bindValue(':start', (int) $start, PDO::PARAM_INT);


            $query->execute();
            $result = $query->fetchAll();

            $response = new CSProResponse(json_encode($result), 200);
        } catch (\Exception $e) {
            $app ['monolog']->addError('Failed getting user list', array("context" => (string) $e));
            $result ['code'] = 500;
            $result ['description'] = 'Failed getting user list';
            $response = new CSProResponse ();
            $response->setError($result ['code'], 'users_get_error', $result ['description']);
        }

        //usersCount
        try {
            $stmCount = 'SELECT COUNT(*) FROM cspro_users';
            $query = $pdo->prepare($stmCount);
            $query->execute();
            $userCount = $query->fetch();
        } catch (\Exception $e) {
            $app ['monolog']->addError('\n\nFailed getting user count', array("context" => (string) $e));
            $result ['code'] = 500;
            $result ['description'] = 'Failed getting user count';
            $response = new CSProResponse ();
            $response->setError($result ['code'], 'users_get_error', $result ['description']);
        }

        //usersFiltered
        try {
            $stmSearchCount = 'SELECT COUNT(*) FROM cspro_users';

            $search = $request->headers->get('x-csw-user-search');

            if ($search != null && $search != "" && $search != " ") {
                $searchStm = " WHERE username LIKE ? OR first_name LIKE ? OR last_name LIKE ? OR email LIKE ? OR phone LIKE ?";
                $stmSearchCount = $stmSearchCount . $searchStm;
            }

            $query = $pdo->prepare($stmSearchCount);
            $query->execute(array("%" . $search . "%", "%" . $search . "%", "%" . $search . "%", "%" . $search . "%", "%" . $search . "%"));
            $usersFiltered = $query->fetch();
        } catch (\Exception $e) {
            $result ['code'] = 500;
            $result ['description'] = 'Failed getting user count';
            $response = new CSProResponse ();
            $response->setError($result ['code'], 'users_get_error', $result ['description']);
            $app ['monolog']->addError('Failed getting user count', array("context" => (string) $e));
        }

        $response->headers->set('Content-Length', strlen($response->getContent()));
        $response->headers->set('x-csw-user-count', $userCount);
        $response->headers->set('x-csw-users-filtered', $usersFiltered);
        return $response;
    }

    function getUser(Application $app, Request $request, $username) {
        $pdo = $app ['pdo'];
        try {
            $stm = 'SELECT username, first_name as firstName, last_name as lastName, email, phone, role as role
				FROM cspro_users where username = :uname';
            $bind = array(
                'uname' => array(
                    'uname' => $username
                )
            );
            $result = $pdo->fetchAll($stm, $bind);
            if (!$result) {
                $response = new CSProResponse ();
                $response->setError(404, 'user_not_found', 'User not found');
                return $response;
            } else {
                $resultUser = $result [0];
                $response = new CSProResponse(json_encode($resultUser));
            }
        } catch (\Exception $e) {
            $app ['monolog']->addError('Failed getting user: ' . username, array("context" => (string) $e));
            $response = new CSProResponse();
            $response->setError(500, 'user_get_error', 'Failed getting user');
        }

        $response->headers->set('Content-Length', strlen($response->getContent()));
        return $response;
    }

    // Update User
    function updateUser(Application $app, Request $request, $username) {
        $params = array();
        $content = $request->getContent();
        $response = new CSProResponse ();

        if (empty($content)) {
            $response->setError(400, null, 'Missing JSON content in the request');
            return $response;
        } else {
            $uri = '#/definitions/User';
            validateEncodedJSON($app, $content, $uri);
            $params = json_decode($content, true); // 2nd param to get as array
            if (!isValidUser($params, false)) {
                $response->setError(400, 'invalid_user', 'Invalid User Supplied. User attributes cannot be blank or invalid.');
                return $response;
            }
        }
        try {
            $pdo = $app ['pdo'];
            $stm = 'SELECT username , role  FROM cspro_users WHERE username = :uname;';
            $bind = array(
                'uname' => array(
                    'uname' => $username
                )
            );
            $result = $pdo->fetchAll($stm, $bind);
            if (count($result) == 0) {
                $response->setError(404, 'user_not_found', 'User not found');
                return $response;
            }
            $userrole = isset($params ['role']) ? $params ['role'] : $result ['role'];
            if (!empty($params ['password'])) {
                $stmt = $pdo->prepare("UPDATE cspro_users
	                               SET username=:uname, password=:pass, first_name=:fname, last_name=:lname, email=:email, phone=:phone, role=:role
	                               WHERE username=:origuname");
            } else {
                $stmt = $pdo->prepare("UPDATE cspro_users
	                               SET username=:uname, first_name=:fname, last_name=:lname, email=:email, phone=:phone, role=:role
	                               WHERE username=:origuname");
            }
            $stmt->bindParam(':uname', $params ['username']);
            if (!empty($params ['password'])) {
                $passwordHash = password_hash($params ['password'], PASSWORD_DEFAULT);
                $stmt->bindParam(':pass', $passwordHash);
            }
            $email  = $params ['email'];
            $phone  = $params ['phone'];
            if (empty(trim($email))) {
               $email = null;
            } 
            if (empty(trim($phone))) {
                $phone = null;
            } 
            $stmt->bindParam(':fname', $params ['firstName']);
            $stmt->bindParam(':lname', $params ['lastName']);
            $stmt->bindParam(':email', $email);
            $stmt->bindParam(':phone', $phone);
            $stmt->bindParam(':origuname', $username);
            $stmt->bindParam(':role', $userrole);
            $stmt->execute();
            $response = new CSProResponse(json_encode(array(
                        "code" => 200,
                        "description" => 'The user ' . $username . ' was successfully updated.'
                    )), 200);
        } catch (\Exception $e) {
            $app ['monolog']->addError('Failed updating user: ' . username, array("context" => (string) $e));
            $result ['code'] = 500;
            $result ['description'] = 'Failed updating user';
            $response = new CSProResponse ();
            $response->setError($result ['code'], 'user_update_failed', $result ['description']);
        }
        $response->headers->set('Content-Length', strlen($response->getContent()));
        return $response;
    }

    // Delete User
    function deleteUser(Application $app, Request $request, $username) {
        $pdo = $app ['pdo'];
        try {
            $pdo->beginTransaction();
            $stm = 'DELETE FROM cspro_users WHERE username = :username';
            $bind = array(
                'username' => array(
                    'username' => $username
                )
            );
            $row_count = $pdo->fetchAffected($stm, $bind);

            if ($row_count == 1) {
                $pdo->commit();
                $result ['code'] = 200;
                $result ['description'] = 'The user ' . $username . ' was successfully deleted.';
                $response = new CSProResponse(json_encode($result));
                $response->headers->set('Content-Length', strlen($response->getContent()));
            } else {
                $result ['code'] = 404;
                $result ['description'] = 'The username ' . $username . ' was not found.';
                $response = new CSProResponse ();
                $response->setError($result ['code'], 'user_delete_failed', $result ['description']);
            }
        } catch (\Exception $e) {
            $pdo->rollBack();
            $app ['monolog']->addError('Failed deleting user: ' . username, array("context" => (string) $e));
            $result ['code'] = 404;
            $result ['description'] = 'The user ' . $username . ' was not deleted.';
            $response = new CSProResponse ();
            $response->setError($result ['code'], 'user_delete_failed', $result ['description']);
        }

        $response->headers->set('Content-Length', strlen($response->getContent()));
        return $response;
    }

    /**
     * Replaces any parameter placeholders in a query with the value of that
     * parameter. Useful for debugging. Assumes anonymous parameters from 
     * $params are are in the same order as specified in $query
     *
     * @param string $query The sql query with parameter placeholders
     * @param array $params The array of substitution parameters
     * @return string The interpolated query
     */
    public function interpolateQuery($query, $params) {
        $keys = array();
        $values = $params;

        # build a regular expression for each parameter
        foreach ($params as $key => $value) {
            if (is_string($key)) {
                $keys[] = '/:' . $key . '/';
            } else {
                $keys[] = '/[?]/';
            }

            if (is_array($value))
                $values[$key] = implode(',', $value);

            if (is_null($value))
                $values[$key] = 'NULL';
        }
        // Walk the array to see if we can add single-quotes to strings
        array_walk($values, create_function('&$v, $k', 'if (!is_numeric($v) && $v!="NULL") $v = "\'".$v."\'";'));

        $query = preg_replace($keys, $values, $query, 1, $count);

        return $query;
    }

}
