<?php

namespace CSPro\User;

use \CSPro\User\User;

class UserValidator {

    private $errors = array();

    public function isValid() {
        return count($this->errors) == 0;
    }

    public function getErrors() {
        return $this->errors;
    }

    public function reset() {
        $this->errors = array();
    }

    public function addError($errMsg) {
        array_push($this->errors, $errMsg);
    }

    // validate user data
    public function validate(User $user, $app = null, $lineNumber = null) {
        $errMsg = "";
        if ((strlen($user->getUserName()) < 4) && !ctype_alnum($user->getUserName())) {
            $errMsg = $errMsg . "Invalid User Name;";
        }

        if (!ctype_alpha($user->getLastName())) {
            $errMsg = $errMsg . "Invalid Last Name;";
        }
        if (!ctype_alpha($user->getFirstName())) {
            $errMsg = $errMsg . "Invalid First Name;";
        }
        if (strlen(trim($user->getPassword())) < 8) {
            $errMsg = $errMsg . "Password must be at least 8 characters;";
        }

        if (!($user->getRole() == "Standard User" || $user->getRole() == "Administrator")) {
            $errMsg = $errMsg . 'Invalid Role Type. Valid role types are: Standard User and Administrator;';
        }

        $email = $user->getEmail();
        if (!empty(trim($email))) {
            $isValidEmail = filter_var($email, FILTER_VALIDATE_EMAIL);
            if ($isValidEmail === false) {
                $errMsg = $errMsg . 'Invalid email address;';
            }
        }

        if (!empty($errMsg)) {
            if (isset($lineNumber)) {
                $errMsg = "Error: " . $errMsg . " Line number:" . $lineNumber . " User:[" . $user->toString() . "]";
            } else {
                $errMsg = "Error: " . $errMsg . " User:[" . $user->toString() . "]";
            }
        }
        if (!empty($errMsg)) {//add the error message if there are errors
            $this->addError($errMsg);
        }

        return empty($errMsg);
    }

    function validateImportUsers($app, $content, $headerRow) {

        //remove any blanks lines
        $content = preg_replace('/^[ \t]*[\r\n]+/m', '', $content);
        $lines = array_filter(explode("\n", $content), 'strlen');

        $user = new User(null, null, null, null, null);
        $this->reset();

        $app['monolog']->addDebug("Number of lines: " . count($lines));

        $lineCount = count($lines);
        if ($headerRow === true)
            $lineCount--;

        if ($lineCount === 0) {
            $this->addError('Import file is empty.');
            return false;
        }

        $app['monolog']->addDebug("Inside Validate Import");
        $startLine = 0;
        if ($headerRow === true) {
            $startLine = 1;
            $app['monolog']->addDebug("Ignoring header row");
        }
        for ($i = $startLine; $i < count($lines); $i++) {

            $csv = str_getcsv($lines[$i], ",", '"', "\n");
            if ($csv[0] == null)
                break;
            //set the user attributes to null if they are not set
            for ($attrIndex = 0; $attrIndex < 7; $attrIndex++) {
                if (!isset($csv[$attrIndex]))
                    $csv[$attrIndex] = null;
            }
            if (empty(trim($csv[5])))
                $csv[5] = null; //email
            if (empty(trim($csv[5])))
                $csv[6] = null; //phone
            $user->setAllMembers($csv[0], $csv[1], $csv[2], $csv[3], $csv[4], $csv[5], $csv[6]);
            $lineNumber = $i + 1;
            $this->validate($user, $app, $lineNumber);
        }
        $isValid = $this->isValid();
        return $isValid;
    }

}
